@@ -260,8 +260,8 @@ reload_image_menu_items (void)
 }
 
 static void
-icon_theme_changed (GnomeIconTheme *icon_theme,
-		    gpointer        data)
+icon_theme_changed (GtkIconTheme *icon_theme,
+		    gpointer      data)
 {
 	reload_image_menu_items ();
 }
@@ -1033,6 +1033,15 @@ menuitem_button_press_event (GtkWidget      *menuitem,
 	return FALSE;
 }
 
+static void  
+drag_begin_menu_cb (GtkWidget *widget, GdkDragContext     *context)
+{
+	/* FIXME: workaround for a possible gtk+ bug
+	 *    See bugs #92085(gtk+) and #91184(panel) for details.
+	 */
+	gtk_tooltips_disable (panel_tooltips);
+}
+
 /* This is a _horrible_ hack to have this here. This needs to be added to the
  * GTK+ menuing code in some manner.
  */
@@ -1115,11 +1124,6 @@ drag_data_get_menu_cb (GtkWidget        *widget,
 				selection_data->target, 8, (guchar *)uri_list,
 				strlen (uri_list));
 	g_free (uri_list);
-
-	/* FIXME: workaround for a possible gtk+ bug
-	 *    See bugs #92085(gtk+) and #91184(panel) for details.
-	 */
-	gtk_tooltips_disable (panel_tooltips);
 }
 
 static void
@@ -1243,9 +1247,10 @@ setup_uri_drag (GtkWidget  *menuitem,
 			     menu_item_targets, 1,
 			     GDK_ACTION_COPY);
 
-	/* FIXME: waiting for bug #116577
-	gtk_drag_source_set_icon_name (GTK_WIDGET (button), icon); */
+	gtk_drag_source_set_icon_name (menuitem, icon);
 	
+	g_signal_connect (G_OBJECT (menuitem), "drag_begin",
+			  G_CALLBACK (drag_begin_menu_cb), NULL);
 	g_signal_connect_data (G_OBJECT (menuitem), "drag_data_get",
 			       G_CALLBACK (drag_data_get_string_cb),
 			       g_strdup (uri),
@@ -1271,12 +1276,12 @@ setup_internal_applet_drag (GtkWidget             *menuitem,
 			     menu_item_targets, 1,
 			     GDK_ACTION_COPY);
 
-	/* FIXME: waiting for bug #116577
 	if (panel_action_get_icon_name (type)  != NULL)
-		gtk_drag_source_set_icon_name (GTK_WIDGET (button),
-					panel_action_get_icon_name (type);
-					*/
+		gtk_drag_source_set_icon_name (menuitem,
+					       panel_action_get_icon_name (type));
 	
+	g_signal_connect (G_OBJECT (menuitem), "drag_begin",
+			  G_CALLBACK (drag_begin_menu_cb), NULL);
 	g_signal_connect_data (G_OBJECT (menuitem), "drag_data_get",
 			       G_CALLBACK (drag_data_get_string_cb),
 			       g_strdup (panel_action_get_drag_id (type)),
@@ -1287,7 +1292,7 @@ setup_internal_applet_drag (GtkWidget             *menuitem,
 }
 
 static void
-load_submenu_to_display (GtkWidget *menu)
+submenu_to_display (GtkWidget *menu)
 {
 	GMenuTree          *tree;
 	GMenuTreeDirectory *directory;
@@ -1315,21 +1320,30 @@ load_submenu_to_display (GtkWidget *menu)
 }
 
 static gboolean
-submenu_to_display (gpointer data)
+submenu_to_display_in_idle (gpointer data)
 {
-	GtkWidget         *menu;
+	GtkWidget *menu = GTK_WIDGET (data);
 
-	menu = GTK_WIDGET (data);
+	g_object_set_data (G_OBJECT (menu), "panel-menu-idle-id", NULL);
 
-	load_submenu_to_display (menu);
+	submenu_to_display (menu);
 
 	return FALSE;
 }
 
+static void
+remove_submenu_to_display_idle (gpointer data)
+{
+	guint idle_id = GPOINTER_TO_UINT (data);
+
+	g_source_remove (idle_id);
+}
+
 static GtkWidget *
 create_fake_menu (GMenuTreeDirectory *directory)
 {	
 	GtkWidget *menu;
+	guint      idle_id;
 	
 	menu = create_empty_menu ();
 
@@ -1347,8 +1361,17 @@ create_fake_menu (GMenuTreeDirectory *directory)
 			   GUINT_TO_POINTER (TRUE));
 
 	g_signal_connect (menu, "show",
-			  G_CALLBACK (load_submenu_to_display), NULL);
-	g_idle_add_full (G_PRIORITY_LOW, submenu_to_display, menu, NULL);
+			  G_CALLBACK (submenu_to_display), NULL);
+
+	idle_id = g_idle_add_full (G_PRIORITY_LOW,
+				   submenu_to_display_in_idle,
+				   menu,
+				   NULL);
+	g_object_set_data_full (G_OBJECT (menu),
+				"panel-menu-idle-id",
+				GUINT_TO_POINTER (idle_id),
+				remove_submenu_to_display_idle);
+
 	g_signal_connect (menu, "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
 
@@ -1478,10 +1501,10 @@ create_menuitem (GtkWidget          *menu,
 				     menu_item_targets, 1,
 				     GDK_ACTION_COPY);
 
-		/* FIXME: waiting for bug #116577
-		gtk_drag_source_set_icon_name (GTK_WIDGET (button),
-					gmenu_tree_entry_get_icon (entry));
-		*/
+		gtk_drag_source_set_icon_name (menuitem,
+					       gmenu_tree_entry_get_icon (entry));
+		g_signal_connect (G_OBJECT (menuitem), "drag_begin",
+				  G_CALLBACK (drag_begin_menu_cb), NULL);
 		g_signal_connect (menuitem, "drag_data_get",
 				  G_CALLBACK (drag_data_get_menu_cb), entry);
 		g_signal_connect (menuitem, "drag_end",
@@ -1557,6 +1580,7 @@ create_applications_menu (const char *menu_file,
 {
 	GMenuTree *tree;
 	GtkWidget *menu;
+	guint      idle_id;
 
 	menu = create_empty_menu ();
 
@@ -1586,8 +1610,17 @@ create_applications_menu (const char *menu_file,
 			   GUINT_TO_POINTER (TRUE));
 
 	g_signal_connect (menu, "show",
-			  G_CALLBACK (load_submenu_to_display), NULL);
-	g_idle_add_full (G_PRIORITY_LOW, submenu_to_display, menu, NULL);
+			  G_CALLBACK (submenu_to_display), NULL);
+
+	idle_id = g_idle_add_full (G_PRIORITY_LOW,
+				   submenu_to_display_in_idle,
+				   menu,
+				   NULL);
+	g_object_set_data_full (G_OBJECT (menu),
+				"panel-menu-idle-id",
+				GUINT_TO_POINTER (idle_id),
+				remove_submenu_to_display_idle);
+
 	g_signal_connect (menu, "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
 
